<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0"
	xmlns="http://www.w3.org/1999/xhtml"
	xmlns:h="http://www.w3.org/1999/xhtml"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:fn="http://www.w3.org/2005/xpath-functions"
	xmlns:svg="http://www.w3.org/2000/svg"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	exclude-result-prefixes="fn h xs">
	<xsl:output 
		method="xml" 
		indent="yes" 
		encoding="UTF-8"		
		doctype-public="-//W3C//DTD XHTML 1.1 plus MathML 2.0 plus SVG 1.1//EN" 
		doctype-system="http://www.w3.org/2002/04/xhtml-math-svg/xhtml-math-svg.dtd"/>
	
		
	<xsl:param name="title"/>
	<xsl:param name="isRoot"/>
	<xsl:param name="urlBase"/>
	<xsl:param name="wwwRead"/>
	<xsl:param name="wwwPost"/>
	
	
	<!-- Celý dokument -->
	<xsl:template match="/">
		<html>
			<head>
				<meta http-equiv="Content-Type" content="application/xhtml+xml; charset=utf-8" />
				<meta http-equiv="X-NNTP-Generated" content="{fn:current-dateTime()}" />
				<xsl:if test="$urlBase">
					<base href="{$urlBase}"/>
				</xsl:if>
				<xsl:if test="$title">
					<title><xsl:value-of select="$title"/></title>
				</xsl:if>
				<style type="text/css">
					body {
						font-family: sans-serif;
						font-size: 16px;
					}
					
					.wwwLinks {
						border-top: 2px solid grey;
						margin-top: 16px;
						font-size: 66%;
					}
					
					blockquote {
						background-color: #eee;
						font-style: italic;
						padding: 2px 20px;
						margin: 10px 32px;
						border-left: 3px solid silver;
					}
					
					pre {
						background-color: #eee;
						padding: 2px 20px;
						margin: 10px 32px;
						border-left: 3px solid #a00;
					}
					
					img {
						margin: 8px;
					}
					
					a img {
						border: none;
					}
					
					/** TODO: smazat */
					.mešuge {
						background-color: #afa;
						border: 1px solid #55f;
					}
				</style>
			</head>
			<body>
				<xsl:if test="$title and $isRoot">
					<h1><xsl:value-of select="$title"/></h1>			
				</xsl:if>
				<xsl:call-template name="zpracujTělo">
					<xsl:with-param name="prvek" select="h:html/h:body/node()[1]"/>
				</xsl:call-template>
				
				<xsl:if test="$wwwRead or $wwwPost">
					<div class="wwwLinks">
						<p>Přístup přes síť www:</p>
						<ul>
							<xsl:if test="$wwwRead"><li><a href="{$wwwRead}">číst tento příspěvek</a></li></xsl:if>
							<xsl:if test="$wwwPost"><li><a href="{$wwwPost}">odpovědět na tento příspěvek</a></li></xsl:if>
						</ul>
					</div>
				</xsl:if>
			</body>
		</html>
	</xsl:template>
	
	
	<!-- Odkazy -->
	<xsl:template match="h:a">
		<a href="{@href}" title="{@title}"><xsl:apply-templates/></a>
	</xsl:template>
	
	
	<!-- Obrázky -->
	<xsl:template match="h:img">
		<img src="{@src}" alt="{@alt}" title="{@title}"><xsl:apply-templates/></img>
	</xsl:template>
	
	<!-- Zkratky -->
	<xsl:template match="h:abbr">
		<abbr title="{@title}"><xsl:apply-templates/></abbr>
	</xsl:template>
	
	
	<!-- Další povolené značky – ostatní odfiltrujeme (zbude z nich jen text) -->
	<xsl:template match="*">
		<xsl:choose>		
			<xsl:when test="
				name() = 'h1' or
				name() = 'h2' or
				name() = 'h3' or
				name() = 'h4' or
				name() = 'h5' or
				name() = 'h6' or
				name() = 'p' or
				name() = 'div' or
				name() = 'br' or
				name() = 'strong' or
				name() = 'em' or
				name() = 'sub' or
				name() = 'sup' or
				name() = 'del' or
				name() = 'ul' or
				name() = 'ol' or
				name() = 'li' or
				name() = 'pre' or
				name() = 'code' or
				name() = 'blockquote'">
				<xsl:element name="{name()}">
					<xsl:apply-templates/>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="."/>
			</xsl:otherwise>
		</xsl:choose>		
    </xsl:template>
    
    <!-- 
    	Z neuzavřeného (nevalidně se vyskytujícího v body) textu uděláme odstavce.
    -->
	<xsl:template name="zpracujTělo">
		<xsl:param name="prvek"/>
		<xsl:param name="vnořeno" select="false()"/>
		<xsl:if test="$prvek">
			<xsl:choose>
				<xsl:when test="$prvek/self::text()">
					<!--
						Textový uzel → budeme dělat odstavce
						(rekurzivně se opět zavolá šablona zpracujTělo)
					-->
					<xsl:call-template name="dělejOdstavce">
						<xsl:with-param name="blokTextu" select="$prvek"/>
						<xsl:with-param name="vnořeno" select="$vnořeno"/>
					</xsl:call-template>
					
					<!-- 
					<xsl:variable name="navázat" as="xs:boolean">
						<xsl:call-template name="navázat">
							<xsl:with-param name="blokTextu" select="$prvek"/>
						</xsl:call-template>
					</xsl:variable>
					
					Někdy zpracujTělo dalšího prvku 
					<xsl:if test="not($navázat)">[další:]
						<xsl:call-template name="zpracujTělo">
							<xsl:with-param name="prvek" select="$prvek/following-sibling::node()[1]"/>
							<xsl:with-param name="vnořeno" select="$vnořeno"/>
						</xsl:call-template>
					</xsl:if>
					-->
					
				</xsl:when>
				<xsl:otherwise>
					<!--
						Jiný uzel → aplikujeme obecné šablony
						a zpracujeme následující uzel.
					-->
					<xsl:apply-templates select="$prvek"/>
					<xsl:call-template name="zpracujTělo">
						<xsl:with-param name="prvek" select="$prvek/following-sibling::node()[1]"/>
						<xsl:with-param name="vnořeno" select="$vnořeno"/>
					</xsl:call-template>
				</xsl:otherwise>					
			</xsl:choose>				
		</xsl:if>
	</xsl:template>
	
	<xsl:variable name="oddělovač" select="'\n\s*\n\s*'"/>
	
	<xsl:template name="navázat" as="xs:boolean">
		<xsl:param name="blokTextu"/>
		<xsl:variable name="dalšíUzel" select="$blokTextu/following-sibling::node()[1]"/>
		<xsl:sequence select="
			not(fn:matches($blokTextu, concat('.*', $oddělovač ,'$'))) and
			$dalšíUzel and
			not(
				$dalšíUzel/name() = 'p' or
				$dalšíUzel/name() = 'div' or								
				$dalšíUzel/name() = 'h1' or								
				$dalšíUzel/name() = 'h2' or								
				$dalšíUzel/name() = 'h3' or								
				$dalšíUzel/name() = 'h4' or								
				$dalšíUzel/name() = 'h5' or								
				$dalšíUzel/name() = 'h6' or								
				$dalšíUzel/name() = 'pre' or								
				$dalšíUzel/name() = 'table' or								
				$dalšíUzel/name() = 'blockquote' or								
				$dalšíUzel/name() = 'hr'								
			)
		"/>
	</xsl:template>
	
	<xsl:template name="dělejOdstavce">
		<xsl:param name="blokTextu"/>
		<xsl:param name="vnořeno" select="false()"/>
		
		<xsl:variable name="dalšíUzel" select="$blokTextu/following-sibling::node()[1]"/>
		
		<xsl:variable name="navázat" as="xs:boolean">
			<xsl:call-template name="navázat">
				<xsl:with-param name="blokTextu" select="$blokTextu"/>
			</xsl:call-template>
		</xsl:variable>
		
		<xsl:for-each select="fn:tokenize($blokTextu, $oddělovač)">
			<!-- TODO: ošetřit prázdné odstavce -->
			<xsl:if test="normalize-space(.) or true()">
				<xsl:choose>
					<xsl:when test="$vnořeno">
						[
							<xsl:value-of select="."/>
							<xsl:if test="$navázat and position() = last()">
								→
								<xsl:call-template name="zpracujTělo">
									<xsl:with-param name="prvek" select="$dalšíUzel"/>
									<xsl:with-param name="vnořeno" select="true()"/>
								</xsl:call-template>
							</xsl:if>
						]
						[Avril:]
						<xsl:if test="not($navázat) and position() = last()">
							a→
								<xsl:call-template name="zpracujTělo">
									<xsl:with-param name="prvek" select="$dalšíUzel"/>
									<xsl:with-param name="vnořeno" select="true()"/>
								</xsl:call-template>
						</xsl:if>
					</xsl:when>
					<xsl:otherwise>
						<p class="mešuge">
						{
							<xsl:value-of select="."/>
							<xsl:if test="$navázat and position() = last()">
								s→
								<xsl:call-template name="zpracujTělo">
									<xsl:with-param name="prvek" select="$dalšíUzel"/>
									<xsl:with-param name="vnořeno" select="true()"/>
								</xsl:call-template>
							</xsl:if>
						}
						</p>
						[sk8:]
						<xsl:if test="not($navázat) and position() = last()">
							→
								<xsl:call-template name="zpracujTělo">
									<xsl:with-param name="prvek" select="$dalšíUzel"/>
									<xsl:with-param name="vnořeno" select="false()"/>
								</xsl:call-template>
						</xsl:if>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:if>
		</xsl:for-each>
		
	</xsl:template>
    
	
</xsl:stylesheet>
